/* 
 * tkMacHLEvents.c --
 *
 *	Implements high level event support for the Macintosh.  Currently, 
 *	the only event that really does anything is the Quit event.
 *
 * Copyright (c) 1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

static char sccsid[] = "@(#) tkMacHLEvents.c 1.6 95/09/08 14:41:30";

#include "tcl.h"
#include "tkMacInt.h"

#include <AppleEvents.h>
#include <SegLoad.h>

static int receivedQuitEvent = false;

static pascal OSErr QuitHandler _ANSI_ARGS_((AppleEvent* event,
	AppleEvent* reply, long refcon));
static pascal OSErr OappHandler _ANSI_ARGS_((AppleEvent* event,
	AppleEvent* reply, long refcon));
static pascal OSErr OdocHandler _ANSI_ARGS_((AppleEvent* event,
	AppleEvent* reply, long refcon));
static pascal OSErr PrintHandler _ANSI_ARGS_((AppleEvent* event,
	AppleEvent* reply, long refcon));

/*
 *----------------------------------------------------------------------
 *
 * TkMacInitAppleEvents --
 *
 *	Initilize the Apple Events on the Macintosh.  This registers the
 *	core event handlers.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

void 
TkMacInitAppleEvents(interp)
    Tcl_Interp *interp;
{
    OSErr err;
    AEEventHandlerUPP	OappHandlerUPP, OdocHandlerUPP,
	PrintHandlerUPP, QuitHandlerUPP;
	
    /* Install event handlers for the core apple events. */
    QuitHandlerUPP = NewAEEventHandlerProc(QuitHandler);
    err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication,
	    QuitHandlerUPP, (long) interp, false);

    OappHandlerUPP = NewAEEventHandlerProc(OappHandler);
    err = AEInstallEventHandler(kCoreEventClass, kAEOpenApplication,
	    OappHandlerUPP, (long) interp, false);

    OdocHandlerUPP = NewAEEventHandlerProc(OdocHandler);
    err = AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments,
	    OdocHandlerUPP, (long) interp, false);

    PrintHandlerUPP = NewAEEventHandlerProc(PrintHandler);
    err = AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments,
	    PrintHandlerUPP, (long) interp, false);

} /* TkMacInitAppleEvents */


/*
 *----------------------------------------------------------------------
 *
 * QuitHandler, OappHandler, etc. --
 *
 *	These are the core Apple event handlers.  Only the Quit event does
 *	anything interesting.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */
static pascal OSErr
QuitHandler(theAppleEvent, reply, handlerRefcon)
    AppleEvent *theAppleEvent;
    AppleEvent *reply;
    long handlerRefcon;
{
    Tcl_Interp 	*interp = (Tcl_Interp *) handlerRefcon;

    receivedQuitEvent = true;
    return noErr;
} /* QuitHandler */


static pascal OSErr
OappHandler(theAppleEvent, reply, handlerRefcon)
    AppleEvent *theAppleEvent;
    AppleEvent *reply;
    long handlerRefcon;
{
    return noErr;
} /* OappHandler */

static pascal OSErr
OdocHandler(theAppleEvent, reply, handlerRefcon)
    AppleEvent *theAppleEvent;
    AppleEvent *reply;
    long handlerRefcon;
{
    return noErr;
} /* OdocHandler */

static pascal OSErr
PrintHandler(theAppleEvent, reply, handlerRefcon)
    AppleEvent *theAppleEvent;
    AppleEvent *reply;
    long handlerRefcon;
{
    return noErr;
} /* PrintHandler */

/*
 *----------------------------------------------------------------------
 *
 * DoHighLevelEvent --
 *
 *	Dispatch incomming highlevel events.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Depends on the incoming event.
 *
 *----------------------------------------------------------------------
 */

void
TkMacDoHLEvent(theEvent)
    EventRecord *theEvent;
{
    OSErr err = noErr;

    /* TODO: currently a bug exists - will not work with powerpc */
    err = AEProcessAppleEvent(theEvent);

    if (receivedQuitEvent == true) {
        ExitToShell();
    }
    
    return;
} /* DoHighLevelEvent */


